from typing import List, Optional

from aws_cdk import core, aws_certificatemanager as acm, aws_route53 as route53
from pydantic import BaseModel, Field

from acru_l.core import Stack, StackFactory
from acru_l.resources.hosted_zone import HostedZone


class HostedZoneFactory(BaseModel):
    name: str
    domain_name: str
    export_name: str
    use_github_pages: bool = False
    github_username: Optional[str] = None
    github_cname: Optional[str] = None

    def build(self, scope: core.Construct):
        return HostedZone(
            scope,
            self.name,
            domain_name=self.domain_name,
            export_name=self.export_name,
            use_github_pages=self.use_github_pages,
            github_username=self.github_username,
            github_cname=self.github_cname,
        )


class CertOptions(BaseModel):
    name: str
    domain_name: str
    export_name: str

    def build(self, scope: core.Construct, hosted_zone: route53.HostedZone):
        cert = acm.DnsValidatedCertificate(
            scope,
            self.name,
            domain_name=self.domain_name,
            hosted_zone=hosted_zone,
        )
        core.CfnOutput(
            scope,
            f"{self.name}Output",
            value=cert.certificate_arn,
            export_name=self.export_name,
        )


class CertsFactory(BaseModel):
    name: Optional[str] = None
    hosted_zone_domain_name: Optional[str] = None
    hosted_zone: Optional[HostedZoneFactory] = None
    certificates: List[CertOptions] = Field(default_factory=list)

    def build(self, scope: core.Construct):
        if self.hosted_zone:
            hosted_zone = self.hosted_zone.build(scope)
        else:
            hosted_zone = route53.HostedZone.from_lookup(
                scope, self.name, domain_name=self.hosted_zone_domain_name
            )

        for cert_options in self.certificates:
            cert_options.build(scope, hosted_zone)


class CertsOptions(BaseModel):
    hosted_zones: List[CertsFactory] = Field(default_factory=list)


class CertificatesStack(Stack):
    def build(self, options: CertsOptions):
        for factory in options.hosted_zones:
            factory.build(self)


class CertificatesStackFactory(StackFactory):
    stack_class = CertificatesStack
    options_class = CertsOptions



from typing import Optional

from aws_cdk import (
    core,
    aws_ec2 as ec2,
    aws_certificatemanager as acm,
    aws_route53 as route53,
)
from pydantic import BaseModel

from acru_l.core import Stack, StackFactory
from acru_l.resources.rds.instances import PostgresInstance, RDSInstanceOptions
from acru_l.services.api.base import ServiceOptions
from acru_l.services.api.wsgi import WSGIService


class LucarioOptions(BaseModel):
    hosted_zone_domain_name: str
    cert_export_name: str
    version: str
    vpc_name: str
    service_options: ServiceOptions
    rds_options: Optional[RDSInstanceOptions]


class LucarioStack(Stack):
    """
    WSGI
    POSTGRES
    LAMBDA
    API GATEWAY

    TODO: add REDIS

    Prerequisites:
    VPC
    HOSTED_ZONE
    CERTIFICATES
    """

    service: WSGIService

    def build(self, options: LucarioOptions):
        vpc = ec2.Vpc.from_lookup(self, "VPC", vpc_name=options.vpc_name)
        hosted_zone = route53.HostedZone.from_lookup(
            self, "HostedZone", domain_name=options.hosted_zone_domain_name
        )

        service_options = options.service_options

        db_options = options.rds_options
        database = None
        if db_options is not None:
            database = PostgresInstance(
                self, "PG", vpc=vpc, options=db_options
            )

            service_options.environment.update(
                {
                    "DB_NAME": database.db_name,
                    "DB_USERNAME": database.db_username,
                    "DB_HOST": database.endpoint_address,
                    "DB_PORT": database.endpoint_port,
                    "DB_SECRET_ID": database.creds.secret_arn,
                }
            )
            service_options.secret_arns += [database.creds.secret_arn]

        # TODO: Add options for Redis Elasticache

        certificate = acm.Certificate.from_certificate_arn(
            self,
            "Certificate",
            core.Fn.import_value(options.cert_export_name),
        )
        self.service = WSGIService(
            self,
            "Service",
            vpc=vpc,
            certificate=certificate,
            hosted_zone=hosted_zone,
            version=options.version,
            deploy_id=self.deploy_id,
            options=service_options,
        )
        # TODO: add dependency from db to service so that the build happens
        #       in the right order.
        # if database:
        #     self.service.allow_connection_to(
        #         database.connection_interface,
        #         ec2.Port.tcp(database.port)
        #     )


class LucarioStackFactory(StackFactory):
    stack_class = LucarioStack
    options_class = LucarioOptions



import json

from aws_cdk import (
    core,
    aws_secretsmanager as secretsmanager,
    aws_ec2 as ec2,
    aws_logs as logs,
    aws_cloudwatch as cloudwatch,
    aws_rds as rds,
)
from pydantic import BaseModel


class RDSInstanceOptions(BaseModel):
    db_name: str
    db_username: str
    add_proxy: bool = False
    multi_az: bool = False
    deletion_protection: bool = False
    setup_alarms: bool = False
    instance_class: str = "BURSTABLE3"
    instance_size: str = "MICRO"

    allocated_storage: int = 20
    port: int = 5432
    removal_policy: core.RemovalPolicy = core.RemovalPolicy.DESTROY
    storage_type: rds.StorageType = rds.StorageType.STANDARD
    storage_encrypted: bool = True
    backup_retention: int = 1
    monitoring_interval: int = 60
    enable_performance_insights: bool = True
    cloudwatch_logs_retention: logs.RetentionDays = (
        logs.RetentionDays.ONE_MONTH
    )  # noqa: E501
    auto_minor_version_upgrade: bool = True

    @property
    def instance_type(self) -> ec2.InstanceType:
        return ec2.InstanceType.of(
            getattr(ec2.InstanceClass, self.instance_class),
            getattr(ec2.InstanceSize, self.instance_size),
        )


class RDSInstance(core.Construct):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        vpc: ec2.Vpc,
        engine: rds.IInstanceEngine,
        options: RDSInstanceOptions,
    ):
        super().__init__(scope, id)
        self.id = id
        self.db_name = options.db_name
        self.db_username = options.db_username

        # Creates a security group for AWS RDS
        sg_rds = ec2.SecurityGroup(
            self,
            "SecurityGroup",
            vpc=vpc,
        )

        # Adds an ingress rule which allows resources in the VPC's CIDR
        # to access the database.
        sg_rds.add_ingress_rule(
            peer=ec2.Peer.ipv4(vpc.vpc_cidr_block),
            connection=ec2.Port.tcp(options.port),
        )
        self.creds = secretsmanager.Secret(
            self,
            "Credentials",
            generate_secret_string=secretsmanager.SecretStringGenerator(
                exclude_punctuation=True,
                include_space=False,
                generate_string_key="password",
                secret_string_template=json.dumps(
                    {"username": options.db_username}
                ),
            ),
        )

        self.instance = rds.DatabaseInstance(
            self,
            "Instance",
            allocated_storage=options.allocated_storage,
            deletion_protection=options.deletion_protection,
            credentials=rds.Credentials.from_password(
                username=options.db_username,
                password=self.creds.secret_value_from_json("password"),
            ),
            security_groups=[sg_rds],
            database_name=options.db_name,
            engine=engine,
            vpc=vpc,
            port=options.port,
            instance_type=options.instance_type,
            removal_policy=options.removal_policy,
            multi_az=options.multi_az,
            storage_type=options.storage_type,
            storage_encrypted=options.storage_encrypted,
            backup_retention=core.Duration.days(options.backup_retention),
            monitoring_interval=core.Duration.seconds(
                options.monitoring_interval
            ),
            enable_performance_insights=options.enable_performance_insights,
            cloudwatch_logs_retention=options.cloudwatch_logs_retention,
            auto_minor_version_upgrade=options.auto_minor_version_upgrade,
        )
        self.port = options.port
        self.proxy = None
        if options.add_proxy:
            self.proxy = self.instance.add_proxy(
                "Proxy",
                secrets=[self.creds],
                vpc=vpc,
                db_proxy_name=f"{scope.stack_name}-{id}-Proxy",
                security_groups=[sg_rds],
            )
        if options.setup_alarms:
            self.setup_alarms()

    @property
    def connection_interface(self):
        return self.proxy or self.instance

    @property
    def vpc(self) -> ec2.Vpc:
        return self.instance.vpc

    @property
    def endpoint_port(self) -> str:
        return self.instance.db_instance_endpoint_port

    @property
    def endpoint_address(self) -> str:
        if self.proxy:
            return self.proxy.endpoint
        return self.instance.db_instance_endpoint_address

    def setup_alarms(self):
        cloudwatch.Alarm(
            self,
            "HighCPU",
            metric=self.instance.metric_cpu_utilization(
                unit=cloudwatch.Unit.PERCENT
            ),
            statistic="Average",
            threshold=80,
            evaluation_periods=2,
            period=core.Duration.minutes(5),
        )

        cloudwatch.Alarm(
            self,
            "HighSwapSpace",
            metric=self.instance.metric(
                "SwapUsage", unit=cloudwatch.Unit.BYTES
            ),
            statistic="Average",
            threshold=209715200,  # 200MB
            evaluation_periods=2,
            period=core.Duration.minutes(5),
            treat_missing_data=cloudwatch.TreatMissingData.NOT_BREACHING,
        )
        cloudwatch.Alarm(
            self,
            "HighReadLatency",
            metric=self.instance.metric(
                "ReadLatency", unit=cloudwatch.Unit.SECONDS
            ),
            statistic="Average",
            threshold=0.05,
            evaluation_periods=2,
            period=core.Duration.minutes(5),
            treat_missing_data=cloudwatch.TreatMissingData.NOT_BREACHING,
        )
        cloudwatch.Alarm(
            self,
            "HighWriteLatency",
            metric=self.instance.metric(
                "WriteLatency", unit=cloudwatch.Unit.SECONDS
            ),
            statistic="Average",
            threshold=0.05,
            evaluation_periods=2,
            period=core.Duration.minutes(5),
            treat_missing_data=cloudwatch.TreatMissingData.NOT_BREACHING,
        )
        cloudwatch.Alarm(
            self,
            "LowFreeStorage",
            metric=self.instance.metric_free_storage_space(
                unit=cloudwatch.Unit.BYTES
            ),
            threshold=1073741824,  # 1GB
            statistic="Average",
            evaluation_periods=2,
            period=core.Duration.minutes(5),
            comparison_operator=cloudwatch.ComparisonOperator.LESS_THAN_OR_EQUAL_TO_THRESHOLD,  # noqa: E501
        )
        cloudwatch.Alarm(
            self,
            "LowFreeMemory",
            metric=self.instance.metric_freeable_memory(
                unit=cloudwatch.Unit.BYTES
            ),
            threshold=104857600,  # 100MB
            evaluation_periods=2,
            period=core.Duration.minutes(5),
            statistic="Average",
            comparison_operator=cloudwatch.ComparisonOperator.LESS_THAN_OR_EQUAL_TO_THRESHOLD,  # noqa: E501
        )


class PostgresInstance(RDSInstance):
    def __init__(self, *args, **kwargs):
        kwargs["engine"] = rds.DatabaseInstanceEngine.postgres(
            version=rds.PostgresEngineVersion.VER_11_5
        )
        super().__init__(*args, **kwargs)



from typing import Mapping, Any, List, Optional

from aws_cdk import (
    core,
    custom_resources as acr,
    aws_ec2 as ec2,
    aws_iam as iam,
    aws_lambda as _lambda,
    aws_lambda_python as lambda_python,
    aws_logs as logs,
)


class CustomResource(core.Construct):

    provider: acr.Provider
    resource: core.CustomResource

    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        on_event_handler: Optional[_lambda.Function],
        policy_statements: Optional[List[iam.PolicyStatement]] = None,
        resource_properties: Optional[Mapping[str, Any]] = None,
    ):
        super().__init__(scope, id)
        self.policy_statements = policy_statements or []
        self.resource_properties = resource_properties or {}
        self.on_event_handler = on_event_handler
        if self.on_event_handler is not None:
            self.setup_resource()

    def setup_resource(self):
        for policy_statement in self.policy_statements:
            self.on_event_handler.add_to_role_policy(policy_statement)

        self.provider = acr.Provider(
            self,
            "Provider",
            on_event_handler=self.on_event_handler,
            log_retention=logs.RetentionDays.ONE_DAY,
        )

        self.resource = core.CustomResource(
            self,
            "Resource",
            service_token=self.provider.service_token,
            properties=self.resource_properties,
        )


class PythonCustomResource(CustomResource):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        source_dir: str = None,
        index: str = "handler.py",
        handler: str = "main",
        runtime: _lambda.Runtime = _lambda.Runtime.PYTHON_3_8,
        layers: Optional[List[_lambda.LayerVersion]] = None,
        environment: Optional[Mapping[str, Any]] = None,
        timeout: core.Duration = core.Duration.seconds(300),
        memory_size: int = 1024,
        log_retention: logs.RetentionDays = logs.RetentionDays.ONE_DAY,
        vpc: Optional[ec2.Vpc] = None,
        **kwargs,
    ):
        kwargs["on_event_handler"] = None
        super().__init__(scope, id, **kwargs)
        environment = environment or {}
        self.on_event_handler = lambda_python.PythonFunction(
            self,
            "OnEventHandler",
            entry=source_dir,
            index=index,
            handler=handler,
            runtime=runtime,
            layers=layers,
            memory_size=memory_size,
            environment=environment,
            timeout=timeout,
            vpc=vpc,
            log_retention=log_retention,
        )
        self.setup_resource()



from aws_cdk import (
    core,
    aws_ec2 as ec2,
)


class VPC(core.Construct):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        name: str,
        cidr: str = "10.1.0.0/16",
        export_name: str = "VpcId"
    ):
        super().__init__(scope, id)

        subnet0 = ec2.SubnetConfiguration(
            name="Public", subnet_type=ec2.SubnetType.PUBLIC, cidr_mask=24
        )
        subnet1 = ec2.SubnetConfiguration(
            name="Private", subnet_type=ec2.SubnetType.PRIVATE, cidr_mask=24
        )
        vpc = ec2.Vpc(
            self,
            name,
            cidr=cidr,
            enable_dns_hostnames=True,
            enable_dns_support=True,
            nat_gateways=1,
            subnet_configuration=[
                subnet0,
                subnet1,
            ],
        )
        # Usage: core.Fn.import_value('VpcId')
        core.CfnOutput(
            self, "VpcId", value=vpc.vpc_id, export_name=export_name
        )



from typing import Optional

from aws_cdk import (
    core,
    aws_route53 as route53,
)


class HostedZone(core.Construct):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        domain_name: str,
        export_name: str = "HostedZoneId",
        use_github_pages: bool = False,
        github_username: Optional[str] = None,
        github_cname: Optional[str] = None,
    ):
        super().__init__(scope, id)

        self.hosted_zone = route53.HostedZone(
            self, "HostedZone", zone_name=domain_name
        )
        # Usage: core.Fn.import_value(export_name)
        core.CfnOutput(
            self,
            "HostedZoneId",
            value=self.hosted_zone.hosted_zone_id,
            export_name=export_name,
        )

        if use_github_pages:
            if (
                github_username
                and github_cname
                and github_cname != domain_name
            ):
                # for subdomains of the apex
                route53.CnameRecord(
                    self,
                    "GithubPagesRecord",
                    zone=self.hosted_zone,
                    record_name=f"{github_cname}.",
                    domain_name=f"{github_username}.github.io.",
                )
            else:
                # for when you want to use the apex domain on github pages
                route53.ARecord(
                    self,
                    "GithubPagesRecord",
                    zone=self.hosted_zone,
                    record_name=f"{domain_name}.",
                    target=route53.RecordTarget.from_ip_addresses(
                        "185.199.108.153",
                        "185.199.109.153",
                        "185.199.110.153",
                        "185.199.111.153",
                    ),
                )
                route53.CnameRecord(
                    self,
                    "WWWRecord",
                    zone=self.hosted_zone,
                    record_name=f"www.{domain_name}.",
                    domain_name=f"{domain_name}.",
                )



import json

from aws_cdk import (
    core,
    aws_secretsmanager as secretsmanager,
    aws_ec2 as ec2,
    aws_rds as rds,
)


class PostgresCluster(core.Construct):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        db_username: str,
        db_name: str,
        vpc: ec2.Vpc,
        stage: str = None,
        auto_pause: bool = True,
        max_capacity: int = 2,
        seconds_util_auto_pause: int = 3600,
        backup_retention_period: int = 1,
        enable_http_endpoint: bool = True,
        deletion_protection: bool = False,
    ):
        super().__init__(scope, id)

        stage = stage or ""
        name = f"{id.lower()}-{stage.lower()}" if stage else f"{id.lower()}"
        identifier = f"{name}-pg-cluster"

        self.db_name = db_name
        self.db_username = db_username

        subnet_group = rds.CfnDBSubnetGroup(
            self,
            f"{id}DBSubnetGroup",
            db_subnet_group_description="API Cluster subnets",
            subnet_ids=[subnet.subnet_id for subnet in vpc.private_subnets],
        )

        self.creds = secretsmanager.Secret(
            self,
            f"{id}Credentials",
            secret_name=identifier,
            generate_secret_string=secretsmanager.SecretStringGenerator(
                exclude_punctuation=True,
                include_space=False,
                generate_string_key="password",
                secret_string_template=json.dumps(
                    {"username": self.db_username}
                ),
            ),
        )
        db_sg = ec2.SecurityGroup(
            self, f"{id}SG", vpc=vpc, allow_all_outbound=True
        )
        db_sg.add_ingress_rule(
            peer=ec2.Peer.ipv4(vpc.vpc_cidr_block),
            connection=ec2.Port.tcp(5432),
        )
        sg_ids = [db_sg.security_group_id]

        self.db_cluster = rds.CfnDBCluster(
            self,
            f"{id}Cluster",
            engine="aurora-postgresql",
            engine_mode="serverless",
            engine_version="10.7",
            enable_http_endpoint=enable_http_endpoint,
            database_name=self.db_name,
            master_username=core.Fn.join(
                "",
                [
                    "{{resolve:secretsmanager:",
                    self.creds.secret_arn,
                    ":SecretString:username}}",
                ],
            ),
            master_user_password=core.Fn.join(
                "",
                [
                    "{{resolve:secretsmanager:",
                    self.creds.secret_arn,
                    ":SecretString:password}}",
                ],
            ),
            backup_retention_period=backup_retention_period,
            scaling_configuration={
                "auto_pause": auto_pause,
                "max_capacity": max_capacity,
                "min_capacity": 2,
                "seconds_util_auto_pause": seconds_util_auto_pause,
            },
            vpc_security_group_ids=sg_ids,
            db_subnet_group_name=subnet_group.ref,
            deletion_protection=deletion_protection,
        )
        self.db_cluster_arn = f"arn:aws:rds:{scope.region}:{scope.account}:cluster:{self.db_cluster.ref}"  # noqa: E501

    @property
    def endpoint_port(self):
        return self.db_cluster.attr_endpoint_port

    @property
    def endpoint_address(self):
        return self.db_cluster.attr_endpoint_address



import boto3
import environs

from acrul_toolkit.custom_resources import CustomResourceEventHandler

env = environs.Env()

ses = boto3.client("ses")
route53 = boto3.client("route53")

CREATE = "CREATE"
UPDATE = "UPSERT"
DELETE = "DELETE"


def manage_ses_domain_validation(action):
    domain = env("DOMAIN")
    hosted_zone_id = env("HOSTED_ZONE_ID")

    verification_token = ses.verify_domain_identity(Domain=domain)[
        "VerificationToken"
    ]
    dkim_tokens = ses.verify_domain_dkim(Domain=domain)["DkimTokens"]
    print("Changing resource record sets")
    changes = [
        {
            "Action": action,
            "ResourceRecordSet": {
                "Name": f"_amazonses.{domain}.",
                "Type": "TXT",
                "TTL": 1800,
                "ResourceRecords": [{"Value": f'"{verification_token}"'}],
            },
        }
    ]
    for dkim_token in dkim_tokens:
        change = {
            "Action": action,
            "ResourceRecordSet": {
                "Name": f"{dkim_token}._domainkey.{domain}.",
                "Type": "CNAME",
                "TTL": 1800,
                "ResourceRecords": [
                    {"Value": f"{dkim_token}.dkim.amazonses.com"}
                ],
            },
        }
        changes.append(change)

    route53.change_resource_record_sets(
        ChangeBatch={"Changes": changes}, HostedZoneId=hosted_zone_id
    )


class EventHandler(CustomResourceEventHandler):
    def on_create(self, event):
        manage_ses_domain_validation(CREATE)

    def on_update(self, event):
        manage_ses_domain_validation(UPDATE)

    def on_delete(self, event):
        manage_ses_domain_validation(DELETE)


on_event = EventHandler()



from aws_cdk import core
from pydantic import BaseModel

from acru_l.core import Stack, StackFactory
from acru_l.resources.cognito import UserPoolOptions, UserPool


class UsersStackOptions(BaseModel):

    user_pool: UserPoolOptions
    export_name: str


class UsersStack(Stack):
    def build(self, options: UsersStackOptions):
        user_pool = UserPool(self, "Pool", options=options.user_pool)
        core.CfnOutput(
            self,
            "UserPoolARNExport",
            value=user_pool.pool.user_pool_arn,
            export_name=options.export_name,
        )


class UsersStackFactory(StackFactory):
    stack_class = UsersStack
    options_class = UsersStackOptions



import os
from typing import Mapping, Any, Optional, List, Dict

from aws_cdk import (
    aws_certificatemanager as acm,
    aws_ec2 as ec2,
    aws_events as events,
    aws_events_targets as targets,
    aws_lambda as _lambda,
    aws_logs as logs,
    aws_route53 as route53,
    aws_s3 as s3,
)
from aws_cdk import core, aws_secretsmanager as secretsmanager
from aws_cdk.aws_ec2 import IConnectable
from pydantic import BaseModel, Field

from acru_l.resources.apigateway import LambdaAPIGateway
from acru_l.resources.canary import Canary
from acru_l.resources.custom_resources import CustomResource
from acru_l.resources.functions import Function


class SecretsOptions(BaseModel):
    name: str
    arn_key: str
    exclude_punctuation: bool = True
    include_space: bool = False
    password_length: int = 32

    def create(self, *, scope: core.Construct) -> secretsmanager.Secret:
        return secretsmanager.Secret(
            scope,
            self.name,
            generate_secret_string=secretsmanager.SecretStringGenerator(
                exclude_punctuation=self.exclude_punctuation,
                include_space=self.include_space,
                password_length=self.password_length,
            ),
        )


class CanaryOptions(BaseModel):

    deploy_id: str
    version: str
    health_check_url: str


class CustomResourceOptions(BaseModel):

    source_path: str
    properties: Mapping[str, Any]


class ServiceOptions(BaseModel):
    domain_name: str
    project_source_path: str
    api_lambda_source_path: Optional[str] = None
    secrets: Optional[List[SecretsOptions]] = Field(default_factory=list)
    secret_arns: Optional[List[str]] = Field(default_factory=list)
    local_environment: Optional[List[str]] = Field(default_factory=list)
    environment: Optional[Dict[str, Any]] = Field(default_factory=dict)
    health_check_url: Optional[str] = None
    pre_deploy_options: Optional[CustomResourceOptions] = None
    post_deploy_options: Optional[CustomResourceOptions] = None


class Service(core.Construct):

    pre_deploy: Optional[CustomResource] = None
    post_deploy: Optional[CustomResource] = None
    canary: Optional[Canary] = None
    layers: List[_lambda.LayerVersion]
    api_lambda: Function
    apigw: LambdaAPIGateway

    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        options: ServiceOptions,
        certificate: acm.Certificate,
        hosted_zone: route53.HostedZone,
        deploy_id: str,
        version: str,
        vpc: Optional[ec2.Vpc] = None,
        function_class=Function,
        runtime: _lambda.Runtime = _lambda.Runtime.PYTHON_3_8,
    ):
        super().__init__(scope, id)
        self.function_class = function_class
        self.private_bucket = s3.Bucket(
            self, "PrivateBucket", public_read_access=False
        )
        self.vpc = vpc
        self.runtime = runtime
        self.environment_variables = options.environment
        self.secret_arns = options.secret_arns

        self.setup_environment(
            secrets=options.secrets,
            local_environment_variables=options.local_environment,
        )
        self.layers = self.package_project(
            source_path=options.project_source_path
        )

        self.add_pre_deploy(options=options.pre_deploy_options)
        self.add_api(
            api_lambda_source_path=options.api_lambda_source_path,
            domain_name=options.domain_name,
            certificate=certificate,
            hosted_zone=hosted_zone,
        )
        canary_options = None
        if options.health_check_url:
            canary_options = CanaryOptions(
                deploy_id=deploy_id,
                version=version,
                health_check_url=options.health_check_url,
            )
        self.add_canary(options=canary_options)
        self.add_post_deploy(options=options.post_deploy_options)

    def setup_environment(
        self,
        *,
        secrets: Optional[List[SecretsOptions]],
        local_environment_variables: Optional[List[str]],
    ):
        secrets = secrets or []
        for secrets_options in secrets:
            _secret = secrets_options.create(scope=self)
            envvar_id = secrets_options.arn_key.upper()
            self.secret_arns.append(_secret.secret_arn)
            self.environment_variables[envvar_id] = _secret.secret_arn

        names = local_environment_variables or []
        for name in names:
            self.environment_variables[name] = os.environ[name]

        bucket_name = self.private_bucket.bucket_name
        self.environment_variables["PRIVATE_S3_BUCKET_NAME"] = bucket_name

    def package_project(
        self, *, source_path: str
    ) -> List[_lambda.LayerVersion]:
        project_layer = _lambda.LayerVersion(
            self,
            "ProjectLayer",
            code=_lambda.Code.from_asset(source_path),
            compatible_runtimes=[self.runtime],
        )
        return [project_layer]

    def make_function(self, name: str, **kwargs):
        return self.function_class(
            self,
            name,
            layers=self.layers,
            private_bucket=self.private_bucket,
            secret_arns=self.secret_arns,
            environment_variables=self.environment_variables,
            vpc=self.vpc,
            runtime=self.runtime,
            **kwargs,
        )

    def add_api(
        self,
        *,
        api_lambda_source_path: str,
        domain_name: str,
        certificate: acm.Certificate,
        hosted_zone: route53.HostedZone,
    ):
        self.api_lambda = self.make_function(
            "MainLambda",
            source_path=api_lambda_source_path,
            profiling=True,
            tracing=_lambda.Tracing.ACTIVE,
            log_retention=logs.RetentionDays.THREE_MONTHS,
        )
        if self.pre_deploy:
            self.api_lambda.handler.node.add_dependency(
                self.pre_deploy.resource
            )

        keep_warm = events.Rule(
            self, "KeepWarm", schedule=events.Schedule.cron(minute="*/5")
        )
        keep_warm.add_target(
            targets.LambdaFunction(handler=self.api_lambda.handler)
        )

        self.apigw = LambdaAPIGateway(
            self,
            "APIGW",
            domain_name=domain_name,
            certificate=certificate,
            handler=self.api_lambda.handler,
            hosted_zone=hosted_zone,
        )

    def add_canary(self, *, options: Optional[CanaryOptions]):
        if options:
            self.canary = Canary(
                self,
                "Canary",
                deploy_id=options.deploy_id,
                version=options.version,
                health_check_url=options.health_check_url,
            )
            self.canary.add_dependency(self.api_lambda.handler)

    def add_pre_deploy(self, *, options: Optional[CustomResourceOptions]):
        if options:
            pre_deploy_lambda = self.make_function(
                "PreDeployLambda", source_path=options.source_path
            )

            self.pre_deploy = CustomResource(
                self,
                "PreDeploy",
                on_event_handler=pre_deploy_lambda.handler,
                resource_properties=options.properties,
            )

    def add_post_deploy(self, *, options: Optional[CustomResourceOptions]):
        if options:
            post_deploy_lambda = self.make_function(
                "PostDeployLambda", source_path=options.source_path
            )

            self.post_deploy = CustomResource(
                self,
                "PostDeploy",
                on_event_handler=post_deploy_lambda.handler,
                resource_properties=options.properties,
            )
            if self.canary:
                self.post_deploy.resource.node.add_dependency(
                    self.canary.resource
                )
            else:
                self.post_deploy.resource.node.add_dependency(
                    self.api_lambda.handler
                )

    def allow_connection_to(self, other: IConnectable, port_range: ec2.Port):
        if self.pre_deploy:
            other.connections.allow_from(
                self.pre_deploy.on_event_handler, port_range
            )
        if self.post_deploy:
            other.connections.allow_from(
                self.post_deploy.on_event_handler, port_range
            )
        other.connections.allow_from(self.api_lambda.handler, port_range)



import os

from apig_wsgi import make_lambda_handler
from acrul_toolkit.module_loading import import_string


WSGI_APPLICATION = os.environ.get("WSGI_APPLICATION")
application = import_string(WSGI_APPLICATION)
handle_request = make_lambda_handler(application, binary_support=True)


def main(event, context):
    if "httpMethod" not in event:
        return {"status": "success"}
    return handle_request(event, context)
